/* Copyright 2017 IBM Corp.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __DIRECT_CONTROLS_H
#define __DIRECT_CONTROLS_H

#include <skiboot.h>
#include <opal.h>
#include <cpu.h>

/* fast reboot APIs */
extern int sreset_all_prepare(void);
extern int sreset_all_others(void);
extern void sreset_all_finish(void);

#endif /* __DIRECT_CONTROLS_H */
