/* Copyright 2017 IBM Corp.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <psr.h>

static int opal_get_power_shift_ratio(u32 handle, int token __unused,
				      u32 *ratio)
{
	if (!ratio || !opal_addr_valid(ratio))
		return OPAL_PARAMETER;

	if (psr_get_class(handle) == PSR_CLASS_OCC)
		return occ_get_psr(handle, ratio);

	return OPAL_UNSUPPORTED;
};

opal_call(OPAL_GET_POWER_SHIFT_RATIO, opal_get_power_shift_ratio, 3);

static int opal_set_power_shift_ratio(u32 handle, int token,
				      u32 ratio)
{
	if (psr_get_class(handle) == PSR_CLASS_OCC)
		return occ_set_psr(handle, token, ratio);

	return OPAL_UNSUPPORTED;
};

opal_call(OPAL_SET_POWER_SHIFT_RATIO, opal_set_power_shift_ratio, 3);
